
/* cglob.c -- written by Istvan Varga, 2003 */

#include <cs.h>
#include "oload.h"

GLOBALS cglob;
static  int     cglob_is_set = 0;

/* prototypes for CsoundAPI functions */

void    *csoundCreate(void *);
int     csoundQueryInterface(const char *, void **, int *);
void    csoundDestroy(void *);
int     csoundGetVersion(void);
void    *csoundGetHostData(void *);
void    csoundSetHostData(void *, void *);
int     csoundPerform(void *, int, char **);
int     csoundCompile(void *, int, char **);
int     csoundPerformKsmps(void *);
int     csoundPerformBuffer(void *);
void    csoundCleanup(void *);
void    csoundReset(void *);
MYFLT   csoundGetSr(void *);
MYFLT   csoundGetKr(void *);
int     csoundGetKsmps(void *);
int     csoundGetNchnls(void *);
int     csoundGetSampleFormat(void *);
int     csoundGetSampleSize(void *);
long    csoundGetInputBufferSize(void *);
long    csoundGetOutputBufferSize(void *);
void    *csoundGetInputBuffer(void *);
void    *csoundGetOutputBuffer(void *);
MYFLT   *csoundGetSpin(void *);
MYFLT   *csoundGetSpout(void *);
MYFLT   csoundGetScoreTime(void *);
MYFLT   csoundGetProgress(void *);
MYFLT   csoundGetProfile(void *);
MYFLT   csoundGetCpuUsage(void *);
int     csoundIsScorePending(void *);
void    csoundSetScorePending(void *, int);
MYFLT   csoundGetScoreOffsetSeconds(void *);
void    csoundSetScoreOffsetSeconds(void *, MYFLT);
void    csoundRewindScore(void *);
void    csoundMessage(void *, const char *, ...);
void    csoundMessageV(void *, const char *, va_list);
void    csoundThrowMessage(void *, const char *, ...);
void    csoundThrowMessageV(void *, const char *, va_list);
void    csoundSetMessageCallback(void *,
                                 void (*)(void *, const char *, va_list));
void    csoundSetThrowMessageCallback(void *, void (*)(void *,
                                                       const char *, va_list));
int     csoundGetMessageLevel(void *);
void    csoundSetMessageLevel(void *, int);
void    csoundInputMessage(void *, const char *);
void    csoundKeyPress(void *, char);
void    csoundSetInputValueCallback(void *, void (*)(void *, char *, MYFLT *));
void    csoundSetOutputValueCallback(void *, void (*)(void *, char *, MYFLT));
void    csoundScoreEvent(void *, char, MYFLT *, long);
void    csoundSetExternalMidiOpenCallback(void *, void (*)(void *));
void    csoundSetExternalMidiReadCallback(void *,
                                          int (*)(void *, unsigned char *,
                                                  int));
void    csoundSetExternalMidiWriteCallback(void *,
                                           int (*)(void *, unsigned char *));
void    csoundSetExternalMidiCloseCallback(void *, void (*)(void *));
int     csoundIsExternalMidiEnabled(void *);
void    csoundSetExternalMidiEnabled(void *, int);
void    csoundSetIsGraphable(void *, int);
void    csoundSetMakeGraphCallback(void *, void (*)(void *, WINDAT *, char *));
void    csoundSetDrawGraphCallback(void *, void (*)(void *, WINDAT *));
void    csoundSetKillGraphCallback(void *, void (*)(void *, WINDAT *windat));
void    csoundSetExitGraphCallback(void *, int (*)(void *));
opcodelist      *csoundNewOpcodeList(void);
void    csoundDisposeOpcodeList(opcodelist *);
int     csoundAppendOpcode(char *, int, int, char *, char *,
                           void (*)(void *), void (*)(void *),
                           void (*)(void *), void (*)(void *));
int     csoundLoadExternal(void *, const char *);
int     csoundLoadExternals(void *);
void    *csoundOpenLibrary(const char *);
void    *csoundCloseLibrary(void *);
void    *csoundGetLibrarySymbol(void *, const char *);
void    csoundSetYieldCallback(void *, int (*)(void *));
void    csoundSetEnv(void *, const char *, const char *);
void    csoundSetPlayopenCallback(void *, void (*)(int, int, float, int));
void    csoundSetRtplayCallback(void *, void (*)(char *, int));
void    csoundSetRecopenCallback(void *, void (*)(int, int, float, int));
void    csoundSetRtrecordCallback(void *, int (*)(char *, int));
void    csoundSetRtcloseCallback(void *, void (*)(void));

/* default GLOBALS structure */

static GLOBALS cglob_ = {
        /*
        * Interface functions.
        */
        csoundGetVersion,
        csoundGetHostData,
        csoundSetHostData,
        csoundPerform,
        csoundCompile,
        csoundPerformKsmps,
        csoundPerformBuffer,
        csoundCleanup,
        csoundReset,
        csoundGetSr,
        csoundGetKr,
        csoundGetKsmps,
        csoundGetNchnls,
        csoundGetSampleFormat,
        csoundGetSampleSize,
        csoundGetInputBufferSize,
        csoundGetOutputBufferSize,
        csoundGetInputBuffer,
        csoundGetOutputBuffer,
        csoundGetSpin,
        csoundGetSpout,
        csoundGetScoreTime,
        csoundGetProgress,
        csoundGetProfile,
        csoundGetCpuUsage,
        csoundIsScorePending,
        csoundSetScorePending,
        csoundGetScoreOffsetSeconds,
        csoundSetScoreOffsetSeconds,
        csoundRewindScore,
        csoundMessage,
        csoundMessageV,
        csoundThrowMessage,
        csoundThrowMessageV,
        csoundSetMessageCallback,
        csoundSetThrowMessageCallback,
        csoundGetMessageLevel,
        csoundSetMessageLevel,
        csoundInputMessage,
        csoundKeyPress,
        csoundSetInputValueCallback,
        csoundSetOutputValueCallback,
        csoundScoreEvent,
        csoundSetExternalMidiOpenCallback,
        csoundSetExternalMidiReadCallback,
        csoundSetExternalMidiWriteCallback,
        csoundSetExternalMidiCloseCallback,
        csoundIsExternalMidiEnabled,
        csoundSetExternalMidiEnabled,
        csoundSetIsGraphable,
        csoundSetMakeGraphCallback,
        csoundSetDrawGraphCallback,
        csoundSetKillGraphCallback,
        csoundSetExitGraphCallback,
        csoundNewOpcodeList,
        csoundDisposeOpcodeList,
        csoundAppendOpcode,
        csoundLoadExternal,
        csoundLoadExternals,
        csoundOpenLibrary,
        csoundCloseLibrary,
        csoundGetLibrarySymbol,
        csoundSetYieldCallback,
        csoundSetEnv,
        csoundSetPlayopenCallback,
        csoundSetRtplayCallback,
        csoundSetRecopenCallback,
        csoundSetRtrecordCallback,
        csoundSetRtcloseCallback,
        /*
        * Data fields.
        */
        NULL            /*  hostData                    */
};

/* reset cglob structure, and install default CsoundAPI callback */
/* functions on first call */

void cglob_RESET(void)
{
    /* reset globals structure */
    if (!cglob_is_set) {
      cglob = cglob_;
      cglob_is_set++;
    }
    else {
      /* if already initialised, preserve function pointers */
      cglob.hostdata = NULL;
    }
}

/* same as cglob_RESET(), but has no effect */
/* if globals are already initialised */

void cglob_init(void)
{
    if (!cglob_is_set) {
      cglob_RESET();
    }
}

