
#ifndef CSOUND_CGLOB_H
#define CSOUND_CGLOB_H

#ifdef __cplusplus
extern "C" {
#endif

typedef struct GLOBALS_
{
    int     (*GetVersion)(void);
    void    *(*GetHostData)(void *csound);
    void    (*SetHostData)(void *csound, void *hostData);
    int     (*Perform)(void *csound, int argc, char **argv);
    int     (*Compile)(void *csound, int argc, char **argv);
    int     (*PerformKsmps)(void *csound);
    int     (*PerformBuffer)(void *csound);
    void    (*Cleanup)(void *csound);
    void    (*Reset)(void *csound);
    MYFLT   (*GetSr)(void *csound);
    MYFLT   (*GetKr)(void *csound);
    int     (*GetKsmps)(void *csound);
    int     (*GetNchnls)(void *csound);
    int     (*GetSampleFormat)(void *csound);
    int     (*GetSampleSize)(void *csound);
    long    (*GetInputBufferSize)(void *csound);
    long    (*GetOutputBufferSize)(void *csound);
    void    *(*GetInputBuffer)(void *csound);
    void    *(*GetOutputBuffer)(void *csound);
    MYFLT   *(*GetSpin)(void *csound);
    MYFLT   *(*GetSpout)(void *csound);
    MYFLT   (*GetScoreTime)(void *csound);
    MYFLT   (*GetProgress)(void *csound);
    MYFLT   (*GetProfile)(void *csound);
    MYFLT   (*GetCpuUsage)(void *csound);
    int     (*IsScorePending)(void *csound);
    void    (*SetScorePending)(void *csound, int pending);
    MYFLT   (*GetScoreOffsetSeconds)(void *csound);
    void    (*SetScoreOffsetSeconds)(void *csound, MYFLT offset);
    void    (*RewindScore)(void *csound);
    void    (*Message)(void *csound, const char *format, ...);
    void    (*MessageV)(void *csound, const char *format, va_list args);
    void    (*ThrowMessage)(void *csound, const char *format, ...);
    void    (*ThrowMessageV)(void *csound, const char *format, va_list args);
    void    (*SetMessageCallback)(void *csound,
                                  void (*csoundMessageCallback)
                                       (void *hostData,
                                        const char *format, va_list valist));
    void    (*SetThrowMessageCallback)(void *csound,
                                       void (*throwMessageCallback)
                                            (void *hostData,
                                             const char *format,
                                             va_list valist));
    int     (*GetMessageLevel)(void *csound);
    void    (*SetMessageLevel)(void *csound, int messageLevel);
    void    (*InputMessage)(void *csound, const char *message_);
    void    (*KeyPress)(void *csound, char c);
    void    (*SetInputValueCallback)(void *csound,
                                     void (*inputValueCalback)
                                          (void *hostData, char *channelName,
                                           MYFLT *value));
    void    (*SetOutputValueCallback)(void *csound,
                                      void (*outputValueCalback)
                                           (void *hostData, char *channelName,
                                            MYFLT value));
    void    (*ScoreEvent)(void *csound, char type, MYFLT *pFields,
                          long numFields);
    void    (*SetExternalMidiOpenCallback)(void *csound,
                                           void (*midiOpenCallback)
                                                (void *hostData));
    void    (*SetExternalMidiReadCallback)(void *csound,
                                           int (*readMidiCallback)
                                               (void *hostData,
                                                unsigned char *midiData,
                                                int size));
    void    (*SetExternalMidiWriteCallback)(void *csound,
                                            int (*writeMidiCallback)
                                                (void *hostData,
                                                 unsigned char *midiData));
    void    (*SetExternalMidiCloseCallback)(void *csound,
                                            void (*closeMidiCallback)
                                                 (void *hostData));
    int     (*IsExternalMidiEnabled)(void *csound);
    void    (*SetExternalMidiEnabled)(void *csound, int enabled);
    void    (*SetIsGraphable)(void *csound, int isGraphable);
    void    (*SetMakeGraphCallback)(void *csound,
                                    void (*makeGraphCallback)(void *hostData,
                                                              WINDAT *p,
                                                              char *name));
    void    (*SetDrawGraphCallback)(void *csound,
                                    void (*drawGraphCallback)
                                         (void *hostData, WINDAT *p));
    void    (*SetKillGraphCallback)(void *csound,
                                    void (*killGraphCallback)(void *hostData,
                                                              WINDAT *p));
    void    (*SetExitGraphCallback)(void *csound, int (*exitGraphCallback)
                                                      (void *hostData));
    opcodelist  *(*NewOpcodeList)(void);
    void    (*DisposeOpcodeList)(opcodelist *opcodelist_);
    int     (*AppendOpcode)(char *opname, int dsblksiz, int thread,
                        char *outypes, char *intypes, SUBR iopadr,
                        SUBR kopadr, SUBR aopadr, SUBR dopadr);
    int     (*LoadExternal)(void *csound, const char *libraryPath);
    int     (*LoadExternals)(void *csound);
    void    *(*OpenLibrary)(const char *libraryPath);
    void    *(*CloseLibrary)(void *library);
    void    *(*GetLibrarySymbol)(void *library, const char *procedureName);
    void    (*SetYieldCallback)(void *csound, int (*yieldCallback)
                                                  (void *hostData));
    void    (*SetEnv)(void *csound, const char *environmentVariableName,
                      const char *path);
    void    (*SetPlayopenCallback)(void *csound,
                                   void (*playopen__)(int nchanls, int dsize,
                                                      float sr, int scale));
    void    (*SetRtplayCallback)(void *csound,
                                 void (*rtplay__)(char *outBuf, int nbytes));
    void    (*SetRecopenCallback)(void *csound,
                                  void (*recopen__)(int nchanls, int dsize,
                                                    float sr, int scale));
    void    (*SetRtrecordCallback)(void *csound,
                                   int (*rtrecord__)(char *inBuf, int nbytes));
    void    (*SetRtcloseCallback)(void *csound, void (*rtclose__)(void));

    void    *hostdata;
} GLOBALS;

extern  GLOBALS cglob;

/* reset cglob structure, and install default CsoundAPI callback */
/* functions on first call */

void cglob_RESET(void);

/* same as cglob_RESET(), but has no effect */
/* if globals are already initialised */

void cglob_init(void);

#ifdef __cplusplus
}
#endif

#endif          /* CSOUND_CGLOB_H */

