#ifndef CS_H            /*                                      CS.H    */
#define CS_H (1)

#ifdef __cplusplus
extern "C" {
#endif

#include <stdarg.h>
#include <stdlib.h>
#include <stdio.h>
#include <setjmp.h>
#include "sysdep.h"
#include "cwindow.h"
#include "opcode.h"

#include "version.h"

#include "csound.h"

#define VMSGS 1
#if VMSGS
#define VMSG(x) if (CSGLOBAL(O).odebug) x
#else
#define VMSG(x)
#endif

#define INSTR     1
#define ENDIN     2
#define OPCODE    3
#define ENDOP     4
#define LABEL     5
#define SETBEG    6
#define STRSET    6
#define PSET      7
#define SETEND    8

#define MAXINSNO 200
#define PMAX     1000
#define VARGMAX  1001
/* IV - Oct 24 2002: max number of input/output args for user defined opcodes */
#define OPCODENUMOUTS   24

#define ORTXT      h.optext->t
#define INCOUNT    ORTXT.inlist->count
#define OUTCOUNT   ORTXT.outlist->count
#define INOCOUNT   ORTXT.inoffs->count
#define OUTOCOUNT  ORTXT.outoffs->count
#define XINCODE    ORTXT.xincod
#  define XINARG1  (p->XINCODE & 2)
#  define XINARG2  (p->XINCODE & 1)
#  define XINARG3  (p->XINCODE & 4)
#  define XINARG4  (p->XINCODE & 8)
#define XOUTCODE   ORTXT.xoutcod        /* IV - Sep 1 2002 */
#define STRARG     ORTXT.strargs[0]
#define STRARG2    ORTXT.strargs[1]
#define STRARG3    ORTXT.strargs[2]
#define STRARG4    ORTXT.strargs[3]

#define MAXLEN     0x1000000L
#define FMAXLEN    ((MYFLT)(MAXLEN))
#define PHMASK     0x0FFFFFFL
#define PFRAC(x)   ((MYFLT)((x) & ftp->lomask) * ftp->lodiv)
#define MAXPOS     0x7FFFFFFFL

#define BYTREVS(n) ((n>>8  & 0xFF) | (n<<8 & 0xFF00))
#define BYTREVL(n) ((n>>24 & 0xFF) | (n>>8 & 0xFF00L)   \
                    | (n<<8 & 0xFF0000L) | (n<<24 & 0xFF000000L))

#define NOCTS      20
#define OCTRES     8192
#define RESMASK    8191L
#define RESHIFT    13
#define CPSOCTL(n) cpsocint[n >> RESHIFT] * cpsocfrc[n & RESMASK]

#define LOBITS     10
#define LOFACT     1024
  /* LOSCAL is 1/LOFACT as MYFLT */
#define LOSCAL     FL(0.0009765625)

#define LOMASK     1023

#define SSTRCOD    0xFFFFFL
#define SSTRSIZ    200
#define ALLCHNLS   0x7fff
#define DFLT_SR    FL(44100.0)
#define DFLT_KR    FL(4410.0)
#define DFLT_KSMPS 10
#define DFLT_NCHNLS 1
#define MAXCHNLS   256

#define MAXNAME 128

#define DFLT_DBFS (FL(32767.0))
void dbfs_init(float dbfs);

typedef struct {
        int     odebug, initonly;
        int     sfread, sfwrite, sfheader, filetyp;
        int     inbufsamps, outbufsamps;
#if defined(LINUX) && defined(HAVE_ALSA)                /* IV - Nov 10 2002 */
        char    *pcm_input_device, *pcm_output_device;  /* ALSA device names */
#endif
        int     informat, outformat;
        int     insampsiz, outsampsiz;
        int     displays, graphsoff, postscript, msglevel;
        int     Beatmode, cmdTempo, oMaxLag;
        char    *outorchnam, *outsconam;
        int     expr_opt;                       /* IV - Jan 08 2003 */
        int     usingcscore, Linein, Midiin, FMidiin;
        int     OrcEvts;        /* - for triginstr (re Aug 1999) */
        int     RTevents, ksensing;
        int     ringbell, termifend, stdoutfd;
        int     rewrt_hdr, heartbeat, gen01defer;
#ifdef LINUX
        int     Volume;         /* Jonathan Mohr  1995 Oct 17 */
#endif
        long    sr_override, kr_override;
        long    instxtcount, optxtsize;
        long    poolcount, gblfixed, gblacount;
        long    argoffsize, strargsize, filnamsize;
        char    *argoffspace, *strargspace, *filnamspace;
        char    *infilename, *outfilename, *playscore;
        char    *Linename, *Midiname, *FMidiname;
#ifdef __BEOS__
        char    *Midioutname; /* jjk 09252000 - MIDI output device, -Q option */
#endif
} OPARMS;

#define  ONEPT          1.021975               /* A440 tuning factor */
#define  LOG10D20       0.11512925             /* for db to ampfac   */
#define  DV32768        FL(0.000030517578125)

typedef struct polish {
        char    opcod[12];
        short   incount;
        char    *arg[4];     /* Was [4][12] */
} POLISH;

typedef struct arglst {
        short   count;
        char    *arg[1];
} ARGLST;

typedef struct argoffs {
        short   count;
        short   indx[1];
} ARGOFFS;

/* Storage for parsed orchestra code, for each opcode in an INSTRTXT. */
typedef struct text {
        short   linenum;    /* Line num in orch file (currently buggy!)      */
        short   opnum;      /* Opcode index in opcodlst[]                    */
        char    *opcod;     /* Pointer to opcode name in global pool         */
        char    *strargs[4];/* (Unquoted) array of file names if opcode uses */
        ARGLST  *inlist;    /* Input args (pointer to item in name list)     */
        ARGLST  *outlist;
        ARGOFFS *inoffs;    /* Input args (index into list of values)        */
        ARGOFFS *outoffs;
        short   xincod;     /* Rate switch for multi-rate opcode functions   */
        short   xoutcod;    /* output rate switch (IV - Sep 1 2002)          */
        char    intype;     /* Type of first input argument (g,k,a,w etc)    */
        char    pftype;     /* Type of output argument (k, a etc)            */
} TEXT;

/* This struct is filled out by otran() at orch parse time.
   It is used as a template for instrument events. */
typedef struct instr {
        struct op * nxtop;          /* Linked list of instr opcodes          */
        TEXT    t;                  /* Text of instrument (same in nxtop)    */
        short   pmax, vmax, pextrab;/* Arg count, size of data for all
                                       opcodes in instr                      */
        short   mdepends;           /* Opcode type (i/k/a)                   */
        short   lclkcnt, lcldcnt;   /* Storage reqs for this instr           */
        short   lclwcnt, lclacnt;
        short   lclpcnt;
        short   lclfixed, optxtcount;
        short   muted;
        long    localen;
        long    opdstot;            /* Total size of opds structs in instr   */
        long    *inslist;           /* Only used in parsing (?)              */
        MYFLT   *psetdata;          /* Used for pset opcode                  */
        struct insds * instance;    /* Chain of allocated instances of
                                       this instrument                       */
        struct insds * lst_instance, *act_instance;     /* IV - Oct 26 2002  */
        struct instr * nxtinstxt;   /* Next instrument in orch (num order)   */
        int     active;             /* To count activations for control      */
        int     maxalloc;
        MYFLT   cpuload;            /* % load this instrumemnt makes         */
        struct opcodinfo *opcode_info;  /* IV - Nov 10 2002 */
        char    *insname;               /* instrument name  */
} INSTRTXT;


/* A chain of TEXT structs. Note that this is identical with the first two
   members of struct INSTRTEXT, and is so typecast at various points in code. */
typedef struct op {
        struct op * nxtop;
        TEXT    t;
} OPTXT;

typedef struct fdch {
        struct fdch * nxtchp;
        int    fd;
} FDCH;

typedef struct auxch {
        struct auxch * nxtchp;
        long   size;
        char   *auxp, *endp;
} AUXCH;

typedef struct monblk {
        short  pch;
        struct monblk *prv;
} MONPCH;

typedef struct {
        int     notnum[4];
} DPEXCL;

typedef struct {
        DPEXCL dpexcl[8];
        int    exclset[75];     /* for keys 25-99 */
} DPARM;

typedef struct dklst {
        struct dklst *nxtlst;
        long   pgmno;
        MYFLT  keylst[1];       /* cnt + keynos */
} DKLST;

#define DKBAS  25

typedef struct mchnblk {
        short  pgmno;
        short  insno;
        short  RegParNo;
        short  mono;
        MONPCH *monobas;
        MONPCH *monocur;
        struct insds *kinsptr[128];
        struct insds *ksusptr[128];
        MYFLT  polyaft[128];
        MYFLT  ctl_val[128];    /* ... with GS vib_rate,, stored in c102-c109 */
        short  ksuscnt;
        short  sustaining;
        MYFLT  aftouch;
        MYFLT  pchbend;
        DKLST  *klists;         /* chain of dpgm keylists */
        DPARM  *dparms;         /* drumset params     */
        int    dpmsb;
        int    dplsb;
} MCHNBLK;

/* This struct holds the info for a concrete instrument event
   instance in performance. */
typedef struct insds {
        struct opds   *nxti;    /* Chain of init-time opcodes */
        struct opds   *nxtp;    /* Chain of performance-time opcodes */
        void    *opcod_iobufs;  /* IV - Jan 08 2003: user opcode I/O buffers */
        int     relesing;       /* Flag to indicate we're releasing, test */
                                /*   with release opcode (IV: changed to int) */
        struct insds
                *nxtinstance;   /* Next allocated instance */
        struct insds
                *prvinstance;   /* Previous allocated instance */
        struct insds  *nxtact;  /* Next in list of active instruments */
        struct insds  *prvact;  /* Previous in list of active instruments */
        struct insds  *nxtoff;  /* Next instrument to terminate */
        FDCH    fdch;       /* Chain of files used by opcodes in this instr */
        AUXCH   auxch;      /* Extra memory used by opcodes in this instr */
        MCHNBLK *m_chnbp;       /* MIDI note info block if event started from */
                                /*   MIDI (otherwise NULL) */
        short   m_pitch;        /* MIDI pitch, for simple access */
        short   m_veloc;        /* ...ditto velocity */
        int     xtratim;        /* Extra release time requested with xtratim */
                                /*   opcode (IV - Jan 08 2003: use int type) */
                                /* (N.B. it is always in global_onedkr units) */
        short   insno;          /* Instrument number */
        short   actflg;         /* Set if instr instance is active */
                                /*   (performing) */
        MYFLT   offbet;         /* Time to turn off event, in score beats */
        MYFLT   offtim;         /* Time to turn off event, in seconds */
                                /*   (negative on indef/tie) */
/* IV - Jan 08 2003: apparently unused */
/*      struct insds  *nxtolap;    ptr to next overlapping voice */
/*      struct GLOBALS_ *csound;   ptr to Csound engine and API for externals */
        void    *opcod_deact;   /* IV - Oct 24 2002 */
        void    *subins_deact;
        MYFLT   p0;             /* Copy of required p-field values for */
        MYFLT   p1;             /* quick access */
        MYFLT   p2;
        MYFLT   p3;
} INSDS;

typedef void    (*SUBR)(void *);

/* This struct holds the info for one opcode in a concrete
   instrument instance in performance. */
typedef struct opds {
        struct opds * nxti;     /* Next opcode in init-time chain */
        struct opds * nxtp;     /* Next opcode in perf-time chain */
        SUBR    iopadr;         /* Initialization (i-time) function pointer */
        SUBR    opadr;          /* Perf-time (k- or a-rate) function pointer */
/*
 * Deinitialization function pointer;
 * if not null, called during cleanup on each opcode instance;
 * useful for deallocating memory or other resources managed by the opcode.
 */
        SUBR    dopadr;         /* Deinitialization function pointer */
        OPTXT   *optext;        /* Orch file template part for this opcode */
        INSDS   *insdshead;     /* Owner instrument instance data structure */
} OPDS;

typedef struct lblblk {
        OPDS    h;
        OPDS    *prvi;
        OPDS    *prvp;
} LBLBLK;

typedef struct oentry {
        char    *opname;
        unsigned short  dsblksiz;
        unsigned short  thread;
        char    *outypes;
        char    *intypes;
        SUBR    iopadr;
        SUBR    kopadr;
        SUBR    aopadr;
/*
 * Deinitialization function pointer;
 * if not null, called during cleanup on each opcode instance;
 * useful for deallocating memory or other resources managed by the opcode.
 */
        SUBR    dopadr;
        void    *useropinfo;    /* IV - Oct 12 2002: user opcode parameters */
        int     prvnum;         /* IV - Oct 31 2002 */
} OENTRY;

typedef struct {
        MYFLT   *begp, *curp, *endp, feedback[6];
        long    scount;
} OCTDAT;

#define MAXOCTS 8

typedef struct {
        long    npts, nocts, nsamps;
        MYFLT   lofrq, hifrq, looct, srate;
        OCTDAT  octdata[MAXOCTS];
        AUXCH   auxch;
} DOWNDAT;

typedef struct {
        long    ktimstamp, ktimprd;
        long    npts, nfreqs, dbout;
        DOWNDAT *downsrcp;
        AUXCH   auxch;
} SPECDAT;

#define AIFF_MAXCHAN 8
typedef struct {
        MYFLT   natcps;
        MYFLT   gainfac;
        short   loopmode1;
        short   loopmode2;
        long    begin1, end1;
        long    begin2, end2;
        MYFLT   fmaxamps[AIFF_MAXCHAN+1];
} AIFFDAT;

typedef struct {
        MYFLT gen01;
        MYFLT ifilno;
        MYFLT iskptim;
        MYFLT iformat;
        MYFLT channel;
        MYFLT sample_rate;
        char  strarg[SSTRSIZ];
} GEN01ARGS;

typedef struct {
        long    flen;
        long    lenmask;
        long    lobits;
        long    lomask;
        MYFLT   lodiv;
        MYFLT   cvtbas, cpscvt;
        short   loopmode1;
        short   loopmode2;
        long    begin1, end1;   /* all these in ..  */
        long    begin2, end2;
        long    soundend, flenfrms; /* .. sample frames */
        long    nchanls;
        long    fno;
        GEN01ARGS gen01args;
        MYFLT   ftable[1];
} FUNC;

typedef struct MEMFIL {
        char    filename[256];  /* Made larger RWD */
        char    *beginp;
        char    *endp;
        long    length;
 struct MEMFIL  *next;
} MEMFIL;

/* This struct holds the data for one score event. */
typedef struct event {
        char    *strarg;        /* Original argument list string of event */
        char    opcod;          /* Event type */
        short   pcnt;           /* Number of p-fields */
        MYFLT   p2orig;         /* Event start time */
        MYFLT   p3orig;         /* Length */
        MYFLT   offtim;         /* k-time to turn off this event */
        MYFLT   p[PMAX+1];      /* All p-fields for this event */
} EVTBLK;

typedef struct eventnode {
        EVTBLK evt; /* Must be first in struct so it can be typecast & freed */
        struct eventnode *nxtevt;
        int    kstart, insno;
} EVTNODE;

typedef struct {
        OPDS    h;
        MYFLT   *ktempo, *istartempo;
        MYFLT   prvtempo;
} TEMPO;

typedef struct opcodinfo {              /* IV - Oct 24 2002 */
        long    instno;
        char    *name, *intypes, *outtypes;
        short   inchns, outchns, perf_incnt, perf_outcnt;
        short   *in_ndx_list, *out_ndx_list;
        INSTRTXT  *ip;
        struct  opcodinfo *prv;
} OPCODINFO;

FUNC   *ftfind(MYFLT*);
MEMFIL *ldmemfile(char *);
#define MAXCHAN       96        /* for 6 ports */

#include "sort.h"
#include <mididefs.h>

#define CSOUND_EXITJMP_NOERROR  (0x1000)

#define ERRSIZ  (200)
#include "globals.h"
#include "cglob.h"

#include "text.h"

#if defined(mac_classic) || defined(SYMANTEC)           /* IV - Jan 30 2003 */
# define POLL_EVENTS() STasks()
# define __cdecl
#else
# define POLL_EVENTS()     (1)
# if (!defined(__BEOS__) && !defined(__CYGWIN__)) || defined(__MWERKS__)
#  define __cdecl
# endif
#endif

/* IV - Jan 30 2003 */
#include "prototyp.h"
#define printf dribble_printf
extern void dribble_printf(const char *fmt, ...);
extern void err_printf(const char *fmt, ...);

#ifdef WIN32
#define tmpnam mytmpnam
char *mytmpnam(char *);
#endif

#ifndef PI
#define PI      (3.14159265358979323846)
#endif
#define TWOPI   (6.28318530717958647692)
#define PI_F    (FL(3.14159265358979323846))
#define TWOPI_F (FL(6.28318530717958647692))


/*
* Move the C++ guards to enclose the entire file,
* in order to enable C++ to #include this file.
*/
#ifdef __cplusplus
};
#endif

#endif /* CS_H */

