
#ifndef CSOUND_GLOBALS_H
#define CSOUND_GLOBALS_H

#ifdef __cplusplus
extern "C" {
#endif

#define CSGLOBAL(x)     __cglob_##x

/* global variables sr, kr, ksmps, and nchnls,         */
/* and some commonly used values calculated from these */

extern  int             CSGLOBAL(ksmps),    CSGLOBAL(global_ksmps);
extern  MYFLT           CSGLOBAL(ensmps),   CSGLOBAL(global_ensmps);
extern  MYFLT           CSGLOBAL(ekr),      CSGLOBAL(global_ekr);
/* 1 / kr */
extern  MYFLT           CSGLOBAL(onedkr),   CSGLOBAL(global_onedkr);
/* 0.5 / kr */
extern  MYFLT           CSGLOBAL(hfkprd),   CSGLOBAL(global_hfkprd);
/* FMAXLEN / kr */
extern  MYFLT           CSGLOBAL(kicvt),    CSGLOBAL(global_kicvt);
/* number of k-periods since beginning of performance */
extern  long            CSGLOBAL(kcounter), CSGLOBAL(global_kcounter);
extern  MYFLT           CSGLOBAL(esr);
/* 1 / sr */
extern  MYFLT           CSGLOBAL(onedsr);
/* FMAXLEN / sr */
extern  MYFLT           CSGLOBAL(sicvt);
extern  int             CSGLOBAL(nchnls);
/* orchestra header variables calculated by otran */
extern  MYFLT           CSGLOBAL(tran_sr), CSGLOBAL(tran_kr);
extern  MYFLT           CSGLOBAL(tran_ksmps);
extern  MYFLT           CSGLOBAL(tran_0dbfs);
extern  int             CSGLOBAL(tran_nchnls);
/* (2PI / sr), (-PI / sr), and (-2PI / sr) */
extern  MYFLT           CSGLOBAL(tpidsr), CSGLOBAL(mpidsr), CSGLOBAL(mtpdsr);

/* orchestra / score names */

extern  char            * CSGLOBAL(orchname), * CSGLOBAL(scorename);
extern  char            * CSGLOBAL(xfilename);

/* float I/O buffers (ksmps samples) */

extern  MYFLT           * CSGLOBAL(spin);
extern  MYFLT           * CSGLOBAL(spout);
extern  int             CSGLOBAL(nspin);
extern  int             CSGLOBAL(nspout);
extern  int             CSGLOBAL(spoutactive);

/* ZAK system */

extern  MYFLT           * CSGLOBAL(zastart);
extern  long            CSGLOBAL(zalast);
extern  MYFLT           * CSGLOBAL(zkstart);
extern  long            CSGLOBAL(zklast);

/* opcode list, user defined opcodes, and named instruments */

extern  OENTRY          * CSGLOBAL(opcodlst);
extern  void            * CSGLOBAL(opcode_list);        /* IV - Oct 31 2002 */
extern  OENTRY          * CSGLOBAL(oplstend);
extern  OPCODINFO       * CSGLOBAL(opcodeInfo);         /* IV - Oct 20 2002 */
extern  void            * CSGLOBAL(instrumentNames);

/* 0dBFS system */

extern  MYFLT           CSGLOBAL(e0dbfs);
extern  MYFLT           CSGLOBAL(dbfs_to_short);
extern  MYFLT           CSGLOBAL(short_to_dbfs);
extern  MYFLT           CSGLOBAL(dbfs_to_float);
extern  MYFLT           CSGLOBAL(float_to_dbfs);
extern  MYFLT           CSGLOBAL(dbfs_to_long);
extern  MYFLT           CSGLOBAL(long_to_dbfs);

/* real-time audio device numbers for Win32 */

extern  unsigned int    CSGLOBAL(rtin_dev);
extern  unsigned int    CSGLOBAL(rtout_dev);

/* for 'midiin' opcode */

extern  int             CSGLOBAL(MIDIINbufIndex);
extern  MIDIMESSAGE     CSGLOBAL(MIDIINbuffer2[MIDIINBUFMAX]);

/* peak amplitude and position, and out of range samples */

extern  MYFLT           CSGLOBAL(maxamp[MAXCHNLS]);     /* note */
extern  MYFLT           CSGLOBAL(smaxamp[MAXCHNLS]);    /* section */
extern  MYFLT           CSGLOBAL(omaxamp[MAXCHNLS]);    /* overall */
extern  MYFLT           * CSGLOBAL(maxampend);
extern  unsigned long   CSGLOBAL(maxpos[MAXCHNLS]);
extern  unsigned long   CSGLOBAL(smaxpos[MAXCHNLS]);
extern  unsigned long   CSGLOBAL(omaxpos[MAXCHNLS]);
extern  long            CSGLOBAL(rngcnt[MAXCHNLS]);
extern  short           CSGLOBAL(rngflg);

extern  short           CSGLOBAL(multichan);    /* non-zero if nchnls > 1 */

/* oload.h (???) */
extern  short           CSGLOBAL(nlabels);
extern  short           CSGLOBAL(ngotos);
extern  int             CSGLOBAL(strsmax);
extern  char            ** CSGLOBAL(strsets);
extern  int             CSGLOBAL(peakchunks);

/* misc. global variables (FIXME: to be documented ... ) */

extern  EVTBLK          * CSGLOBAL(currevent);
extern  int             CSGLOBAL(keep_tmp);
extern  int             CSGLOBAL(dither_output);
extern  FILE            * CSGLOBAL(dribble);            /* log file */
extern  long            CSGLOBAL(holdrand);
extern  int             CSGLOBAL(maxinsno);
extern  int             CSGLOBAL(maxopcno);             /* IV - Oct 24 2002 */
extern  INSDS           * CSGLOBAL(curip);
extern  EVTBLK          * CSGLOBAL(Linevtblk);
extern  long            CSGLOBAL(nrecs);
extern  FILE            * CSGLOBAL(Linepipe);
extern  int             CSGLOBAL(Linefd);
extern  MYFLT           * CSGLOBAL(ls_table);
extern  MYFLT           CSGLOBAL(curr_func_sr);
extern  char            * CSGLOBAL(retfilnam);
extern  INSTRTXT        ** CSGLOBAL(instrtxtp);
/* sprintf space for compiling msgs */
extern  char            CSGLOBAL(errmsg[ERRSIZ]);
extern  FILE            * CSGLOBAL(scfp);
extern  FILE            * CSGLOBAL(oscfp);
extern  int             CSGLOBAL(tieflag);
extern  char            * CSGLOBAL(ssdirpath), * CSGLOBAL(sfdirpath);
extern  char            * CSGLOBAL(tokenstring);
extern  POLISH          * CSGLOBAL(polish);
extern  FILE            * CSGLOBAL(scorein);
extern  FILE            * CSGLOBAL(scoreout);
extern  MYFLT           * CSGLOBAL(pool);
extern  short           * CSGLOBAL(argoffspace);
extern  INSDS           * CSGLOBAL(frstoff);
extern  int             CSGLOBAL(sensType);
extern  SRTBLK          * CSGLOBAL(frstbp);
extern  int             CSGLOBAL(sectcnt);
extern  MCHNBLK         * CSGLOBAL(m_chnbp[MAXCHAN]);
extern  MYFLT           * CSGLOBAL(cpsocint), * CSGLOBAL(cpsocfrc);
extern  int             CSGLOBAL(inerrcnt);         /* init error count */
extern  int             CSGLOBAL(synterrcnt);       /* syntax error count */
extern  int             CSGLOBAL(perferrcnt);       /* perf error count */
extern  int             CSGLOBAL(MIDIoutDONE);
extern  int             CSGLOBAL(midi_out);
extern  char            CSGLOBAL(strmsg[100]);
extern  INSTRTXT        CSGLOBAL(instxtanchor);
extern  INSDS           CSGLOBAL(actanchor);
/* List of started events, used in playevents() */
extern  EVTNODE         CSGLOBAL(OrcTrigEvts);
extern  char            CSGLOBAL(name_full[256]);   /* Remember name used */
extern  int             CSGLOBAL(Mforcdecs), CSGLOBAL(Mxtroffs);
extern  int             CSGLOBAL(MTrkend);
extern  char            * CSGLOBAL(sadirpath);
extern  char            * CSGLOBAL(oplibs);
extern  OPARMS          * CSGLOBAL(oparms);
extern  jmp_buf         CSGLOBAL(exitjmp);

extern  OPARMS          CSGLOBAL(O);

/* define macros for easier access from Csound sources */

#define ksmps           CSGLOBAL(ksmps)
#define esr             CSGLOBAL(esr)
#define ekr             CSGLOBAL(ekr)
#define global_ksmps    CSGLOBAL(global_ksmps)
#define global_ensmps   CSGLOBAL(global_ensmps)
#define global_ekr      CSGLOBAL(global_ekr)
#define global_onedkr   CSGLOBAL(global_onedkr)
#define global_hfkprd   CSGLOBAL(global_hfkprd)
#define global_kicvt    CSGLOBAL(global_kicvt)
#define global_kcounter CSGLOBAL(global_kcounter)
#define nchnls          CSGLOBAL(nchnls)
#define nlabels         CSGLOBAL(nlabels)
#define ngotos          CSGLOBAL(ngotos)
#define strsets         CSGLOBAL(strsets)
#define strsmax         CSGLOBAL(strsmax)
#define peakchunks      CSGLOBAL(peakchunks)
#define zastart         CSGLOBAL(zastart)
#define zalast          CSGLOBAL(zalast)
#define zkstart         CSGLOBAL(zkstart)
#define zklast          CSGLOBAL(zklast)
#define kcounter        CSGLOBAL(kcounter)
#define currevent       CSGLOBAL(currevent)
#define onedkr          CSGLOBAL(onedkr)
#define onedsr          CSGLOBAL(onedsr)
#define kicvt           CSGLOBAL(kicvt)
#define sicvt           CSGLOBAL(sicvt)
#define spin            CSGLOBAL(spin)
#define spout           CSGLOBAL(spout)
#define nspin           CSGLOBAL(nspin)
#define nspout          CSGLOBAL(nspout)
#define spoutactive     CSGLOBAL(spoutactive)
#define keep_tmp        CSGLOBAL(keep_tmp)
#define dither_output   CSGLOBAL(dither_output)
#define opcodlst        CSGLOBAL(opcodlst)
#define opcode_list     CSGLOBAL(opcode_list)   /* IV - Oct 31 2002 */
#define oplstend        CSGLOBAL(oplstend)
#define dribble         CSGLOBAL(dribble)
#define holdrand        CSGLOBAL(holdrand)
#define maxinsno        CSGLOBAL(maxinsno)
#define maxopcno        CSGLOBAL(maxopcno)      /* IV - Oct 24 2002 */
#define curip           CSGLOBAL(curip)
#define Linevtblk       CSGLOBAL(Linevtblk)
#define nrecs           CSGLOBAL(nrecs)
#define Linepipe        CSGLOBAL(Linepipe)
#define Linefd          CSGLOBAL(Linefd)
#define ls_table        CSGLOBAL(ls_table)
#define curr_func_sr    CSGLOBAL(curr_func_sr)
#define retfilnam       CSGLOBAL(retfilnam)
#define orchname        CSGLOBAL(orchname)
#define scorename       CSGLOBAL(scorename)
#define xfilename       CSGLOBAL(xfilename)
#define instrtxtp       CSGLOBAL(instrtxtp)
#define errmsg          CSGLOBAL(errmsg)
#define scfp            CSGLOBAL(scfp)
#define oscfp           CSGLOBAL(oscfp)
#define maxamp          CSGLOBAL(maxamp)
#define smaxamp         CSGLOBAL(smaxamp)
#define omaxamp         CSGLOBAL(omaxamp)
#define maxampend       CSGLOBAL(maxampend)
#define maxpos          CSGLOBAL(maxpos)
#define smaxpos         CSGLOBAL(smaxpos)
#define omaxpos         CSGLOBAL(omaxpos)
#define tieflag         CSGLOBAL(tieflag)
#define ssdirpath       CSGLOBAL(ssdirpath)
#define sfdirpath       CSGLOBAL(sfdirpath)
#define tokenstring     CSGLOBAL(tokenstring)
#define polish          CSGLOBAL(polish)
#define SCOREIN         CSGLOBAL(scorein)
#define SCOREOUT        CSGLOBAL(scoreout)
#define ensmps          CSGLOBAL(ensmps)
#define hfkprd          CSGLOBAL(hfkprd)
#define pool            CSGLOBAL(pool)
#define ARGOFFSPACE     CSGLOBAL(argoffspace)
#define frstoff         CSGLOBAL(frstoff)
#define sensType        CSGLOBAL(sensType)
#define frstbp          CSGLOBAL(frstbp)
#define sectcnt         CSGLOBAL(sectcnt)
#define M_CHNBP         CSGLOBAL(m_chnbp)
#define cpsocint        CSGLOBAL(cpsocint)
#define cpsocfrc        CSGLOBAL(cpsocfrc)
#define inerrcnt        CSGLOBAL(inerrcnt)
#define synterrcnt      CSGLOBAL(synterrcnt)
#define perferrcnt      CSGLOBAL(perferrcnt)
#define MIDIoutDONE     CSGLOBAL(MIDIoutDONE)
#define midi_out        CSGLOBAL(midi_out)
#define strmsg          CSGLOBAL(strmsg)
#define instxtanchor    CSGLOBAL(instxtanchor)
#define actanchor       CSGLOBAL(actanchor)
#define rngcnt          CSGLOBAL(rngcnt)
#define rngflg          CSGLOBAL(rngflg)
#define multichan       CSGLOBAL(multichan)
#define OrcTrigEvts     CSGLOBAL(OrcTrigEvts)
#define name_full       CSGLOBAL(name_full)
#define Mforcdecs       CSGLOBAL(Mforcdecs)
#define Mxtroffs        CSGLOBAL(Mxtroffs)
#define MTrkend         CSGLOBAL(MTrkend)
#define tran_sr         CSGLOBAL(tran_sr)
#define tran_kr         CSGLOBAL(tran_kr)
#define tran_ksmps      CSGLOBAL(tran_ksmps)
#define tran_0dbfs      CSGLOBAL(tran_0dbfs)
#define tran_nchnls     CSGLOBAL(tran_nchnls)
#define tpidsr          CSGLOBAL(tpidsr)
#define mpidsr          CSGLOBAL(mpidsr)
#define mtpdsr          CSGLOBAL(mtpdsr)
#define sadirpath       CSGLOBAL(sadirpath)
#define opcodeInfo      CSGLOBAL(opcodeInfo)    /* IV - Oct 20 2002 */
#define instrumentNames CSGLOBAL(instrumentNames)
#define e0dbfs          CSGLOBAL(e0dbfs)
#define dbfs_to_short   CSGLOBAL(dbfs_to_short)
#define short_to_dbfs   CSGLOBAL(short_to_dbfs)
#define dbfs_to_float   CSGLOBAL(dbfs_to_float)
#define float_to_dbfs   CSGLOBAL(float_to_dbfs)
#define dbfs_to_long    CSGLOBAL(dbfs_to_long)
#define long_to_dbfs    CSGLOBAL(long_to_dbfs)
#define rtin_dev        CSGLOBAL(rtin_dev)
#define rtout_dev       CSGLOBAL(rtout_dev)
#define MIDIINbufIndex  CSGLOBAL(MIDIINbufIndex)
#define MIDIINbuffer2   CSGLOBAL(MIDIINbuffer2)
#define exitjmp         CSGLOBAL(exitjmp)

/* reset all global variables */

void globalsRESET(void);

/* same as globalsRESET(), but has no effect */
/* if globals are already initialised */

void globals_init(void);

#ifdef __cplusplus
}
#endif

#endif          /* CSOUND_GLOBALS_H */

