
/* main.c -- written by Istvan Varga, 2003 */

#include <cs.h>

extern  void    remove_tmpfiles(void);
extern  void    set_rt_priority(int, char**);

static  int     run_cnt = 1;         /* times to run */

int main(int argc, char **argv)
{
    void  *csnd;
    int   retval;

    /* Real-time priority on Linux by Istvan Varga (Jan 6 2002) */
    /* This function is called before anything else to avoid    */
    /* running "normal" Csound code with setuid root.           */
    /* set_rt_priority gives up root privileges after setting   */
    /* priority and locking memory; init_getstring () and other */
    /* functions below will be run as a normal user (unless     */
    /* Csound was actually run as root, and not setuid root).   */

#if defined(LINUX)
    set_rt_priority(argc, argv);
#endif

    atexit(remove_tmpfiles);
    /* debugging flag for testing re-entrant operation */
    if (argc > 1) {
      if (!strncmp(argv[1], "--reentry-cnt=", 14)) {
        sscanf(argv[1] + 14, "%d", &run_cnt);
        if (run_cnt < 1) run_cnt = 1;
        /* skip option */
        argc--; argv++;
      }
    }
    do {
      /* initialise */
      csnd = csoundCreate(NULL);
      if (csnd == NULL)
        return -1;
      /* perform */
      retval = csoundPerform(csnd, argc, argv);
      if (retval)       /* in case of error:     */
        return retval;  /* terminate immediately */
      /* delete instance */
      csoundDestroy(csnd);
    } while (--run_cnt);

    return 0;
}

