
/* utility stub - written by Istvan Varga, 2003 */
/* emulates stand-alone Csound utilities by     */
/* calling main Csound executable               */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <errno.h>
#include <ctype.h>
#include <unistd.h>

/* default Csound executable */
#ifdef WIN32
char    default_csnd[] = "csoundfltk";
#else
char    default_csnd[] = "csound";
#endif

char    cs_fullname[256], cs_dirname[256], cs_basename[256];
char    util_dirname[256], util_basename[256];

/* split filename to directory and base name */

void split_filename(char *fullname, char *dir, char *bas)
{
    int n, m;
    /* if no filename was given */
    if (fullname == NULL || strlen(fullname) == 0) {
      *dir = '\0';
      *bas = '\0';
      return;
    }
    n = m = strlen(fullname);
    while (--m >= 0 &&
           fullname[m] != '/' && fullname[m] != '\\' && fullname[m] != ':');
    /* directory name */
    if (m < 0)
#ifdef WIN32
      strcpy(dir, ".\\");
#else
      strcpy(dir, "./");
#endif
    else {
      strncpy(dir, fullname, m + 1);
      dir[m + 1] = '\0';
    }
    /* base name */
    strcpy(bas, fullname + (m + 1));
}

int main(int argc, char **argv)
{
    int     i;
    char    *s, *cs_argv[256];

    if (argc < 1)
      return -1;
    /* find Csound executable */
    s = getenv("CSOUND");               /* from environment variable */
    if (s == NULL) s = default_csnd;    /* or use default */
    strcpy(cs_fullname, s);
    /* split to directory and file name */
    split_filename(cs_fullname, cs_dirname, cs_basename);
    /* utility name - same as program name */
    split_filename(argv[0], util_dirname, util_basename);
    /* remove any extension, and convert to lower case */
    if ((s = strchr(util_basename, '.')) != NULL)
      *s = '\0';
    for (s = util_basename; *s != '\0'; s++)
      if (isupper(*s)) *s = tolower(*s);
    if (util_basename[0] == '\0') {
      fprintf(stderr, "error: no utility name\n");
      return(-1);
    }
    /* create command line */
    cs_argv[0] = cs_basename;
    cs_argv[1] = "-U";
    cs_argv[2] = util_basename;
    for (i = 1; i < argc; i++)
      cs_argv[i + 2] = argv[i];
    cs_argv[i + 2] = NULL;              /* terminate argument list */
    /* execute command */
    if (execvp(cs_fullname, cs_argv)) {
      fprintf(stderr, "error executing Csound command: %s: %s\n",
                      cs_fullname, strerror(errno));
      exit(-1);
    }

    return 0;
}

